#ifndef DLL_SYSTEM_ACCESS_H_INCLUDED
#define DLL_SYSTEM_ACCESS_H_INCLUDED


#include <stdlib.h>
#include <stdint.h>


class C_DllIpAddr;
class C_DllSet;
class C_DllSocket;

typedef void*  t_DllThreadHandle;
typedef void*  t_DllSemHandle;
typedef void*  t_DllQueueHandle;
typedef void*  t_DllMutexHandle;

typedef void (*t_DllThreadEntryFctn) (void* UserParam);


#define DLL_ORIG_AUTH_INFO_BUFF_SIZE 256
    
typedef struct t_DllOrigAuthInfo
{
   unsigned char buffer[DLL_ORIG_AUTH_INFO_BUFF_SIZE];
} t_DllOrigAuthInfo;



// t_DllExceptionHandler shall always return 1.
// !! notice: function is directly called by exception handler and must be non-blocking !
typedef int (*t_DllExceptionHandler) (unsigned IrqStatus, void* User);

typedef enum t_DLL_IRQ_ID
{
	DLL_IRQ_DMA_INT_VEC,
	DLL_IRQ_GPIO_EXT_SEL,
	
	DLL_IRQ_COUNT
} t_DLL_IRQ_ID;

enum t_DLL_PROT_ID
{
	DLL_PROT_UDP_V4 = 0,
	DLL_PROT_UDP_V6 = 1,
	DLL_PROT_TCP_V4 = 2,
	DLL_PROT_TCP_V6 = 3,
	DLL_PROT_TLS_V4 = 4,
	DLL_PROT_TLS_V6 = 5
};

#define DLL_SYS_FOREVER 0xFFFFFFFF
#define DLL_SYS_NO_WAIT 0

// definitions of different print-channels on telnet+com
// stdterm is always printed; the other IDs are only visible if the corresponding
// print-channel is activated using e.g. #print syscon
enum t_DLL_PRINT_ID
{
	DLL_STDTERM    = 0,
	DLL_IMAGEPIPE,
	DLL_SYSCON,
	DLL_AUTOFOCUS,
	DLL_SXI,
	DLL_VCA,

	DLL_PRINT_ID_LAST
};

//////////////////////////////////////////////////////////////////////////////
///
/// This class contains the methods for specified system access from a Dll
///
class C_DllSystemAccess
{
	public:
	
	#define DLL_SYSTEM_ACCESS_MAJOR_VERSION 1
	
	struct t_Version
	{
		unsigned int Major; // version number of C_DllSystemAccess
		unsigned int Minor; // version number of C_DllSystemAccessExt_xxx (derived class) or 0 if there is no derivation
	};
	
//	C_DllSystemAccess(){};
//	virtual ~C_DllSystemAccess() {}; // adding the virtual destructor will change the VFT although it might be useful to have a virtual destructor (deletion may call wrong destructor)

	
	/** read the 90kHz clock */
	virtual unsigned GetTime90KHz();


	// TCP/UDP functions:
	// 04/2015: OBSOLETE and replaced by dummy functions to keep virtual function table structure the same
	// please use function 'createSocket' below and C_DllSocket from dll_sock.h instead
	
	virtual int obsolete1() {return -1;};
	virtual int obsolete2() {return -1;};
	virtual int obsolete3() {return -1;};
	virtual int obsolete4() {return -1;};
	virtual int obsolete5() {return -1;};
	virtual int obsolete6() {return -1;};
	virtual int obsolete7() {return -1;};
	virtual int obsolete8() {return -1;};
	virtual int obsolete9() {return -1;};
	virtual int obsolete10() {return -1;};
	virtual int obsolete11() {return -1;};
	virtual int obsolete12() {return -1;};
	virtual int obsolete13() {return -1;};
	virtual int obsolete14() {return -1;};
	virtual int obsolete15() {return -1;};
	virtual int obsolete16() {return -1;};
	virtual int obsolete17() {return -1;};
	virtual int obsolete18() {return -1;};
	virtual int obsolete19() {return -1;};
	virtual int obsolete20() {return -1;};


	// Threading 
	
	/** Create a new Thread
	 * \param szName			name of thread
	 * \param pEntryFctn		pointer to function which is to be the new thread
	 * \param UserParam		function parameter(s)
	 * \return					thread handle */
	virtual t_DllThreadHandle createThread(char *szName, t_DllThreadEntryFctn pEntryFctn, void* UserParam);
	
	/** Delete a Thread
	 * \param hThread			thread handle */
	virtual void deleteThread(t_DllThreadHandle hThread);
	
	/** Sleep within a thread for a given period of time
	 * Note: When a thread function contains a loop and no Semaphore is
	 *			used within this loop, some sleep time must be included. 
	 *			Otherwise the task uses the complete system performance and does
	 *			not hand over to other tasks.
	 * \param Ticks10ms		time to sleep in 10ms */
	virtual void sleepThread(unsigned Ticks10ms);

	/** Check if thread is still running (or if it has been deleted already)
	 * Note: This check should be included in every loop within a thread function 
	 *			(e.g. while(threadRunning() > 0) {}). Otherwise a thread cannot
	 *			be deleted/closed properly!
	 * \return 					1 if still running, 0 if not running, negative in case of error */
	virtual int threadRunning();
				
	
	// Semaphores
	
	/** Create a new Semaphore
	 * \param szName				name of semaphore
	 * \param dwInitialCount	initial count
	 * \return 						semaphore handle */
	virtual t_DllSemHandle createSemaphore(char *szName, unsigned dwInitialCount);
	
	/** Delete a Semaphore
	 * \param hSem					semaphore handle */
	virtual void deleteSemaphore(t_DllSemHandle hSem);
	
	/** Get/Wait for a Semaphore
	 * \param hSem					semaphore handle
	 * \param dwWaitOption		timeout (value in 10ms or DLL_SYS_FOREVER)
	 * \return 						1 if successful, 0 if still waiting for semaphore, negative if function call failed */
	virtual int getSemaphore(t_DllSemHandle hSem, unsigned dwWaitOption);

	/** Put Semaphore
	 * \param hSem					semaphore handle
	 * \return 						1 if successful, 0 if semaphore does not exist, negative if function call failed */
	virtual int putSemaphore(t_DllSemHandle hSem);
	
	
	// Interrupts
	
	/** Register Interrupts
	 * \param irqID				ID of IRQ to register
	 * \param h						function pointer to be called by exception handler. Notice: function must be non-blocking!
	 * \param User					user parameter passed from exception handler when calling function 'h'
	 * \return 						1 if successful, 0 if irqId could not be set */
	virtual int setExceptionHandler(t_DLL_IRQ_ID irqId, t_DllExceptionHandler h, void* User, const char * szName);
	
	/** Enable interrupt
	 * \param id					Interrupt ID
	 * \return 						1 if successful, 0 if IRQ could not be enabled (unknown id)*/
	virtual int enableIRQ(t_DLL_IRQ_ID id);

	/** Disable interrupt
	 * \param id					Interrupt ID
	 * \return 						1 if successful, 0 if IRQ could not be disabled (unknown id)*/
	virtual int disableIRQ(t_DLL_IRQ_ID id);
	
	/** Set interrupt polarity
	 * \param id					Interrupt ID
	 * \param ActiveHigh			1=High Active or rising edge / 0=LowActive or falling edge 
	 * \return 						1 if successful, 0 if polarity could not be set (unknown id)*/
	virtual int irqSetPolarity(t_DLL_IRQ_ID id, int ActiveHigh);
	
	/** Set interrupt level mode
	 * \param id					Interrupt ID
	 * \param LevelMode			1= level triggered / 0= edge triggered
	 * \return 						1 if successful, 0 if level mode could not be set (unknown id)*/
	virtual int irqSetLevelMode(t_DLL_IRQ_ID id, int LevelMode);
	

	// Queues

	/** Create a new WORD * CQueue
	 * \param QueueName			Queue name
	 * \param QueueSize			Define queue size
	 * \return 						Handle of CQueue, if NULL creating failed*/
	virtual t_DllQueueHandle CQueueNewPWord(char * QueueName, unsigned int QueueSize);
	
	/** Enqueue WordPtr
	 * \param Handle				Handle of a CQueue
	 * \param Data					Ptr which should be added to the queue
	 * \param WaitTime			Max wait time (in 10ms steps -> WaitTime=10 => 100ms)
	 * \return 						1 successfully added the element to the queue / 0  Error*/
	virtual unsigned int CQueueEnQueuePWord(t_DllQueueHandle Handle, unsigned short *Data,  unsigned int WaitTime);
	
	/** Dequeue WordPtr
	 * \param Handle				Handle of a CQueue
	 * \param Data					Returns the dequeued data pointer
	 * \param WaitTime			Max wait time (in 10ms steps -> WaitTime=10 => 100ms)
	 * \return 						1 successfully dequeued a element / 0  Error*/
	virtual unsigned int CQueueDeQueuePWord(t_DllQueueHandle Handle, unsigned short **Data, unsigned int WaitTime);
	
	/** Dequeue WordPtr
	 * \param Handle				Handle of a CQueue
	 * \param Enqueued			Returns the fill level of the queue
	 * \return 						1 successfully dequeued a element / 0  Error*/
	virtual unsigned int CQueueEnqueuedPWord(t_DllQueueHandle Handle, unsigned int &Enqueued);
	
	/** Create a new void * CQueue
	 * \param QueueName			Queue name
	 * \param QueueSize			Define queue size
	 * \return 						Handle of CQueue, if NULL creating failed*/
	virtual t_DllQueueHandle CQueueNew(char * QueueName, unsigned int QueueSize);
	
	/** Enqueue VoidPtr
	 * \param Handle				Handle of a CQueue
	 * \param Data					Ptr which should be added to the queue
	 * \param WaitTime			Max wait time (in 10ms steps -> WaitTime=10 => 100ms)
	 * \return 						1 successfully added the element to the queue / 0  Error*/
	virtual unsigned int CQueueEnQueue(t_DllQueueHandle Handle, void* Data,  unsigned int WaitTime);
	
	/** Dequeue VoidPtr
	 * \param Handle				Handle of a CQueue
	 * \param Data					Returns the dequeued data pointer
	 * \param WaitTime			Max wait time (in 10ms steps -> WaitTime=10 => 100ms)
	 * \return 						1 successfully dequeued a element / 0  Error*/
	virtual unsigned int CQueueDeQueue(t_DllQueueHandle Handle, void* &Data, unsigned int WaitTime);

	/** Delete a WORD * CQueue
	 * \param Handle				Handle of a CQueue
	 * \return 						1 successful  / 0  error */
	virtual unsigned int CQueueDeletePWord(t_DllQueueHandle Handle);

	/** Delete a void * CQueue
	 * \param Handle				Handle of a CQueue
	 * \return 						1 successful  / 0  error */
	virtual unsigned int CQueueDelete(t_DllQueueHandle Handle);


	// print

	/** print to telnet + com port
	 * \param id		 		defines the corresponding print channel
	 * \param fmt, ...		string that should be printed; can optionally contain format specifiers
	 * 							which are replaced by values of subsequent arguments (same as standard 'printf')
	 */
	virtual int printf_id(t_DLL_PRINT_ID id, char *fmt, ...);


	// Network (C++ interface, TLS); see also "dll_sock.h"
	
	/** Creates a new socket object and returns a pointer to the created object.
	 * \param prot    specifies the type/protocol for the created socket (see declaration of class C_DllSocket for supported values) 
	 * \return			Socket object if successful, else NULL */
	virtual C_DllSocket *createSocket(t_DLL_PROT_ID prot);
	
	/** Deletes the given socket object and frees the allocated memory.
	 * \param pSock   pointer to the socket object to be deleted
	 * \return			true if successful, else false */
	virtual bool deleteSocket(C_DllSocket *pSock);


	// memory allocation

	/** Allocates memory on the heap
	 * \param size    memory size to be allocated
	 * \return			pointer to the allocated memory */
	virtual void* malloc(size_t size);
	
	/** Frees memory that was allocated with malloc()
	 * \param p       pointer to memory that should be freed
	 * \return			void */
	virtual void free(void* p);
	

	// version info

	/** Get version of C_DllSystemAccess to check if the given object and the interface header are compatible
	 * \return			version number struct*/
	virtual struct t_Version getVersion();
	
	
	// Network (Ip address); see also "dll_sock.h"
	
	/** Creates a new IP address object and returns a pointer to the created object.
	 * \param pIpString  IPv4 or IPv6 address given as string; if NULL an empty IP address object is created
	 * \return			   IP address object if successful, else NULL */
	virtual C_DllIpAddr* createIpAddr(char const* pIpString);
	
	/** Deletes the given IP address object and frees the allocated memory. 
	 * \param pAddr      pointer to the IP address object to be deleted
	 * \return			   true if successful, else false */
	virtual bool deleteIpAddr(C_DllIpAddr* pAddr);
	
	/** Creates a new IP address object in the given memory area
	 * \param pData      pointer to the memory area where the object should be created
	 * \param nBytes     size of the memory area in Bytes
	 * \param pIpString  IPv4 or IPv6 address given as string; if NULL an empty IP address object is created
	 * \return			   IP address object if successful, else NULL */
	virtual C_DllIpAddr* createIpAddr(void* pData, size_t nBytes, char const* pIpString);
	
	
	// Network (socket set); see also "dll_sock.h"
	
	/** Creates a new socket set object and returns a pointer to the created object.
	 * \return			   socket set object if successful, else NULL */
	virtual C_DllSet* createSockSet();
	
	/** Deletes the given socket set object and frees the allocated memory. 
	 * \param pSet       pointer to the socket set object to be deleted
	 * \return			   true if successful, else false */
	virtual bool deleteSockSet(C_DllSet* pSet);
	
	/** Creates a new socket set object in the given memory area
	 * \param pData      pointer to the memory area where the object should be created
	 * \param nBytes     size of the memory area in Bytes
	 * \return			   socket set object if successful, else NULL */
	virtual C_DllSet* createSockSet(void* pData, size_t nBytes);
	
	/** print to telnet + com port
	 * \param buf       target buffer
	 * \param max_bytes buffer len 
	 * \param fmt, ...  string that should be printed; can optionally contain format specifiers
	 *                  which are replaced by values of subsequent arguments (same as standard 'printf')
	 * \return          int nr bytes written */
	virtual int sprintfEx(char *buf, size_t max_bytes, const char *fmt, ...);

	// Recursive Mutex 
	// applied as dummy 1-4
	
	/** Create a new mutex.
	 *  This is a counting mutex. The thread holding the mutex can reget it as often as it wants.
	 *  The thread must put the mutex the same amount as it has got it. Then the mutex is free
	 *  for other threads again.
	 *  Getting the mutex from a thread not owning it at the moment will block the thread until the owning
	 *  thread has returned all the allocations it has.
	 * \param szName				name of mutex
	 * \return 						mutex handle */
	virtual t_DllMutexHandle createMutex(char *szName);
	
	/** Delete a mutex
	 * \param hMutex				mutex handle */
	virtual void deleteMutex(t_DllMutexHandle hMutex);
	
	/** Get/Wait for a mutex
	 * \param hMutex				mutex handle
	 * \param dwWaitOption			timeout (value in 10ms or DLL_SYS_FOREVER)
	 * \return 						1 if successful, 0 if still waiting for mutex, negative if function call failed */
	virtual int getMutex(t_DllMutexHandle hMutex, unsigned dwWaitOption);

	/** Put mutex
	 * \param hMutex				mutex handle
	 * \return 						1 if successful, 0 if mutex does not exist, negative if function call failed */
	virtual int putMutex(t_DllMutexHandle hMutex);

   /* 
    * Parameters to be used with syslogStatsRegisterParameter() below
    */

   #define SLST_UNUSED   0
   #define SLST_NUM_MAX  1    // BitField: track max value (min and max can be combined in one parameter !!!)
   #define SLST_NUM_MIN  2    // BitField: track min value (min and max can be combined in one parameter !!!)
   #define SLST_NUM_SUM  4    // BitField: sum all reported values. e.g. operation hour counter. Not to be combined with min/max
   #define SLST_MAX_NAME_LEN 40
   
   typedef struct t_SlsSetupIfc
   {                     
      unsigned int ParamType;              // type of registered param e.g.:
                                           //   ParamType = (SLST_NUM_MAX)                  // Accumulate Max values
                                           //   ParamType = (SLST_NUM_MIN | SLST_NUM_MAX)   // Accumulate Min and Max values
                                           //   ParamType = (SLST_NUM_SUM)                  // Sum all reported values. 
                                                                                            // e.g. OperationHours
      bool ExportToGui;                    // True if the parameter should be displayed at enduser GUI
      unsigned char Reserved10;            // Must be zero !!
      unsigned char Reserved11;            // Must be zero !!
      unsigned char Reserved12;            // Must be zero !!
      unsigned int Reserved2;              // Must be zero !!
      char ParamName[SLST_MAX_NAME_LEN];   // Systemwide unique ID and Display name. e.g. "OperationHours"
      char UnitFormat[SLST_MAX_NAME_LEN];  // Display format string                  e.g. "%3.1f C"
                                           //                                         or  "%3.1f Hours"
      bool SyslogReport;                   // enable syslog messages on threshold exceeded 
      double InitialValue;                 // Livetime inital value for Min/Max values e.g. 23.0 for an Min/Max temperatur value
      double AlarmHiThreshold;             // For Max values, if SyslogReport is TRUE a message sent once this threshold is exceeded
      double AlarmLoThreshold;             // For Min values, ditto
   }  t_SlsSetupIfc;
	 
   /*
    * Register a new entry for persistent system logging. e.g. min/max values, livetime-sum, operation-hours ...
    * The paramter must be registered once on bootup (always with same unique(!) name and same type)
    * On success the function returns a param handle > -1 which can be used to call syslogStatsReportValue() below
    */
	virtual int syslogStatsRegisterParameter(t_SlsSetupIfc *pParam);

   /*
    * Function to report an update on an persisten system statistics value
    * The parameter hast to be registered one after bootup using syslogStatsRegisterParameter() before it can be uesed.
    * The value returned by syslogStatsRegisterParameter() has to be used as Hdl here.
    * the value of Hdl MUST be retrieved new on EACH bootup.
    * The way how "Value" is accumulated depends on the setting defined in pParam during registering the parameter.
    *
    */
	virtual int syslogStatsReportValue(unsigned int Hdl, double Value);


   /*
    * Function to report persistent system events to be shown on the normal users settings webpage
    *   severity is one of the values defined in syslog rfc 5424 and can be used to apply a filter threshold which can be adjusted by the user.
    *   format and var-args are meant to supply a human readable single line message string in English language without any formatting (e.g. HTML, Color etc.)
    *
    */                     
   
   // Syslog Severity levels RFC 5424 
   #define SLS_EMERGENCY 0 
   #define SLS_ALERT     1
   #define SLS_CRITICAL  2
   #define SLS_ERROR     3
   #define SLS_WARNING   4
   #define SLS_NOTICE    5
   #define SLS_INFO      6
   #define SLS_DEBUG     7

   // moduleID for syslog()
   #define SLM_SC_LOG         0     // System controller events for enduser logging table
   #define SLM_SC_ASA_REP     1     // System controller events for internal ASA-Report
   #define SLM_IMG_PIPE_LOG   2     // Image-Pipe
   #define SLM_CONFIG_SEALING 3     // Report changes related to config-sealing (callers OrigAuthInfo has to have a valid value before)
   
	virtual void syslog(int severity, int moduleID, const char* format, ...);
	
	
   /* 
    * All configuration changes needs to be tracked in the system log with the name of the related user.
    * The subsequent 3 functions are for this purpose
    */ 

   
   /* 
    * Function to get the current authentication information from the current thread
    * All threads have a authentication information which tells on which account they are currently running.
    * - If a thread is only a deamon then it runs on its own local daemon account.
    * - If a thread performs jobs initiated by a real external user then it (should) run with OrigAuthInfo set to the account of this user
    *   as long as the job for this user takes so all actions taken by this execution-path can be tracked with the real user name.
    */
	virtual int getOrigAuthInfo(t_DllOrigAuthInfo *pAuthInfoBuff);
	
	
	/* This function temporary set the authentication information of the current thread to a new account.
    * This should be applied always before actions are performed which might end up in a subsequent configuration change.
	 * It is always needed where jobs from a command queue are processed since the system is not capable to know on its own when
	 * a user based action is queued by one thread and executed by another.
	 * So the enqueuing thread need to store its onw AuthInfo into the job queue-element and the dequeuing thread
	 * needs to write the queued info to its onw context for the period of executing this job.
	 *
	 * After job is done the thread needs to call dropOrigAuthInfo() to return to its prior account.
	 * Thread AutInfo is a stack which will overflow if drop is not always preformed.
	 **
	 * ATTENTION => Only a reference to the supplied object is added => the referenced object mut be keeped valid until dropOrigAuthInfo() is called
	 */
	virtual int addOrigAuthInfo(const t_DllOrigAuthInfo *p_OrigAuthInfo);
	
	
   /*
      Remove the active temporary account from the current threads AuthInfo stack and return to the prior (or base) account
   */
	virtual int dropOrigAuthInfo(const t_DllOrigAuthInfo *p_OrigAuthInfo);
	

   /*
      setupOrigAuthInfo() is intended to "forge" a custom authentication for logging and config sealing
      To get an authentication for a daemon, use the following example:
        => setupOrigAuthInfo((&Oai, NULL, NULL, DLL_OAI_USER_LOCAL_DAEMON);
   */	
   #define DLL_OAI_USER_UNKNOWN              0  // default
   #define DLL_OAI_USER_LOCAL_DAEMON       -42  // a local system daemon which run on its own. (e.g. recording sceduler) with service rights (NOT allowd for proxies like HTTP, RTSP etc.)
   #define DLL_OAI_USER_DERIVE_FROM_THREAD -41  // indicates that the info should be derived from the calling thread instead from this info
     
	virtual int setupOrigAuthInfo(t_DllOrigAuthInfo *p_OrigAuthInfo, const char *pUser, const char *pServer, int LocalUserId);
	
   /* 
    *  Similar to addOrigAuthInfo() but this function puts a copy of the supplied info into the current thread
    *  Only a singe base value can be set (no recursive stack add,add,add,drop,drop,drop as on addOrigAuthInfo() is supported)
    *  In contrast to addOrigAuthInfo() the external object must not be maintained since all data are copied
    *  subsequent calls to setOrigAuthInfo() will simply overwrite the content while addOrigAuthInfo() does stacking.
    */	
	virtual int setOrigAuthInfo(const t_DllOrigAuthInfo *p_OrigAuthInfo);

	/*
	 * spare entries in our vtable for later extensions
	 * (Some dll access classes are derived from this one. So adding new functions here would shift the vtable entries of those classes.
	 *  To avoid this, replace below dummy functions if adding new functions. Then the vtable layout stays the same.)
	 */          
	 
	
	virtual bool dummy13(){return false;};
	virtual bool dummy14(){return false;};
	virtual bool dummy15(){return false;};
	virtual bool dummy16(){return false;};
	virtual bool dummy17(){return false;};
	virtual bool dummy18(){return false;};
	virtual bool dummy19(){return false;};
	virtual bool dummy20(){return false;};
};


#endif // DLL_SYSTEM_ACCESS_H_INCLUDED
